package diego

import diego.aop.UsesDependencyService
import diego.init.Settings
import diego.web.Router
import diego.web.StaticHandlerOptions
import groovy.transform.NamedVariant

/**
 * The new descriptor for applications.
 *
 * @author Kevin Sheppard
 * Date: Feb 27, 2023
 */
abstract class Application implements UsesDependencyService {

  void onStart() {}

  /**
   *
   * @param options Options to pass to the StaticHandler
   * @return A new instance of {@link StaticHandlerOptions}
   */
  @NamedVariant(autoDelegate = true)
  static StaticHandlerOptions serve(StaticHandlerOptions options) {
    return options
  }

  /**
   * Serve a directory using smart defaults depending on the application environment.
   *
   * @param dir The directory to serve (relative to CWD)
   * @return A new instance of {@link StaticHandlerOptions}
   */
  static StaticHandlerOptions serve(String dir) {
    return StaticHandlerOptions.create(dir, Settings.instance.env)
  }

  abstract void configure(Router router)

}