package diego

import com.google.inject.Guice
import com.google.inject.Injector
import com.google.inject.Module
import diego.utils.Timer
import groovy.util.logging.Slf4j
/**
 * A dependency manager for all of the framework components.
 *
 * @author Kevin Sheppard
 * Date: Nov 02, 2021
 */
@Slf4j
class Components {
  private static Injector injector

  static void initializeFramework() {

  }

  /**
   * Use the injector to create an instance of Type T.
   *
   * @param type The type of class to create.
   * @return An instance of T.
   */
  static <T> T createInstance(Class<T> type) {
    if(!isInitialized())
      throw new RuntimeException("The Injector has not been initialized.")
    return injector.getInstance(type)
  }

  /**
   *
   * @return True if the Injector has been initialized.
   */
  static boolean isInitialized() {
    return injector != null
  }

  /**
   * Create an injector using specific modules.
   *
   * @param modules The collection of Modules with which to build the injector.
   */
  static void initializeWith(Module... modules) {
    final var timer = new Timer().startClock()
    injector = Guice.createInjector(modules)
    log.trace("Injector created in {}s", timer.stopClock().elapsedTime)
  }

  /**
   * Empty the dependency container.
   */
  static void clear() {
    injector = null
  }
}
