package diego.app;

/**
 * <p>During the course of the application's life cycle, it will emit events. This class returns an instance of the type T
 * as its "detail".</p>
 *
 * <p>This allows us to do things like:</p>
 *
 * <p>{@code app.addEventListener(APPLICATION_START, e -> printf("Host is %s", e.detail) )}</p>
 *
 * @author Kevin Sheppard
 * Date: Feb 24, 2022
 */
public interface Event<T> {
  /**
   * Users can use this static field when using the addEventListener(String, EventListener)
   */
  public static final String APPLICATION_START = "start";
  public static final String APPLICATION_FAIL = "fail";
  public static final String APPLICATION_STATUS_CHANGE = "status-change";

  T getDetail();
}
