package diego.db;

import diego.db.collections.EntityCollection;

import java.io.Serializable;
import java.util.Map;

/**
 * A very neat abstraction for the common database operations I've been using for the past ~8 years or so.
 */
public interface DAO {
    /**
     * Create a native SQL query.
     *
     * @param sql A native SQL query.
     * @return An SqlQuery instance.
     */
    SqlQuery createQuery(String sql);

    /**
     * Create a native SQL query with positional parameters
     *
     * @param sql A native SQL statement
     * @param positionalParams Positional parameters
     * @return An SqlQuery instance
     */
    SqlQuery createQuery(String sql, Object... positionalParams);

    /**
     *
     * @param sql A native SQL query.
     * @param namedParams a Map of named parameters.
     * @return An SqlQuery instance.
     */
    SqlQuery createQuery(String sql, Map<String, Object> namedParams);

    /**
     * Create an {@link EntityCollection} for a particular type.
     *
     * @param entityType The annotated @Entity type
     * @param idType The type of the @Id field on the entity
     * @return A {@link diego.db.collections.EntityCollectionImpl}
     * @param <T> Class ot type T
     * @param <K> Class of type K which extends Serializable
     */
    <T, K extends Serializable> EntityCollection<T, K> collection(Class<T> entityType, Class<K> idType);

}
