package diego.db;

import com.google.inject.ImplementedBy;
import org.hibernate.Session;

import java.io.Closeable;
import java.util.function.Consumer;

/**
 * A class you can use to work with the database directly instead of creating a service. Useful for
 * scheduled jobs and anything that happens outside the request/response lifecycle.
 */
@ImplementedBy(DatabaseImpl.class)
public interface Database extends DAO, Closeable {
    /**
     * Get the current Session from the {@link SessionManager}. One will be created if it does not exist.
     *
     * @return The active Session for this thread.
     */
    Session getSession();
}
