package diego.db

import com.google.inject.Inject
import com.google.inject.Singleton
import groovy.util.logging.Slf4j

import javax.persistence.Id
import java.beans.Introspector
import java.beans.PropertyDescriptor

/**
 * @author Kevin Sheppard
 * Date: Feb 5, 2022
 */
@Singleton
@Slf4j
class DatabaseImpl extends Service implements Database {

    @Inject
    DatabaseImpl(SessionManager sessionManager) {
        this.sessionManager = sessionManager
    }

    /**
     *
     * Leaving this here as I might want to use it to enhance the PropertyMapper later on.
     */
    protected static Serializable extractId(Object instance) {
        var klass = instance.getClass()
        while (klass != Object) {
            var fields = klass.declaredFields
            var field = Arrays.asList(fields)
                    .stream().filter({ it.isAnnotationPresent(Id) && it.type instanceof Serializable })
                    .findFirst()
            if (field.present) {
                var fieldName = field.get().name
                for (final PropertyDescriptor pd : Introspector.getBeanInfo(klass).propertyDescriptors) {
                    if (pd.name == fieldName && pd.readMethod) return pd.readMethod.invoke(instance) as Serializable
                }
            } else {
                klass = klass.superclass
            }
        }
        throw new RuntimeException("No field with @${Id.name} annotation found for class ${klass.name}.")
    }

    void close() throws IOException {
        sessionManager.close()
    }
}
