package diego.db

import com.google.inject.AbstractModule
import com.google.inject.Scopes
import com.google.inject.matcher.Matchers
import diego.aop.NamedMethodMatcher
import diego.db.collections.EntityCollection
import diego.web.HttpRequestHandler
import groovy.util.logging.Slf4j
import org.hibernate.SessionFactory

import javax.transaction.Transactional
/**
 * Stores all of the logic required to set up the database for dependency injection.
 *
 * @author Kevin Sheppard
 * Date: Feb 23, 2022
 */
@Slf4j
class DatabaseModule extends AbstractModule {
    private final SessionFactory sessionFactory

    DatabaseModule(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory
    }

    @Override
    protected void configure() {
        super.configure()
        final var sessionManager = new SessionManager(sessionFactory)
        final var transactionalMethodInterceptor = new TransactionalMethodInterceptor(sessionManager)
        [Service, SqlQuery, EntityCollection].each { klass ->
            bindInterceptor(Matchers.subclassesOf(klass),
                    Matchers.annotatedWith(Transactional),
                    transactionalMethodInterceptor)
        }
        bindInterceptor(Matchers.subclassesOf(HttpRequestHandler), NamedMethodMatcher.forMethodName('handle'), new SessionManagerInterceptor(sessionManager))
        bind(SessionFactory).toInstance(sessionFactory)
        bind(SessionManager).toInstance(sessionManager)
    }
}
