package diego.db


import diego.Components
import diego.aop.ApplicationDependencyContainerService
import diego.aop.DependencyContainerService
import diego.db.collections.EntityCollection
import diego.db.collections.EntityCollectionImpl

/**
 * @author Kevin Sheppard
 * Date: July 07, 2023
 */
abstract class Service implements DAO, SessionStore {
    protected DependencyContainerService dependencyContainerService = new ApplicationDependencyContainerService()

    SqlQuery createQuery(String sql) {
        return query(sql)
    }

    SqlQuery createQuery(String sql, Map<String, Object> namedParams) {
        return query(sql).params(namedParams)
    }

    SqlQuery createQuery(String sql, Object... positionalParams) {
        return query(sql).params(positionalParams)
    }

    <T, K extends Serializable> EntityCollection<T, K> collection(Class<T> entityType, Class<K> keyType) {
        return dependencyContainerService.getBean(EntityCollectionImpl).forTypes(entityType, keyType)
    }

    private SqlQueryImpl query(String sql) {
        return new SqlQueryImpl(session, sql)
    }
}
