package diego.db

import com.google.inject.Inject
import org.aopalliance.intercept.MethodInterceptor
import org.aopalliance.intercept.MethodInvocation

/**
 * Closes the active Session after every response has been sent.
 *
 * @author Kevin Sheppard
 * Date: Aug 17, 2022
 */
class SessionManagerInterceptor implements MethodInterceptor {
  private final SessionManager sessionManager

  SessionManagerInterceptor(SessionManager sessionManager) {
    this.sessionManager = sessionManager
  }

  @Override
  Object invoke(MethodInvocation invocation) throws Throwable {
    Object result = null
    try {
      result = invocation.proceed()
    } finally {
      sessionManager.close()
      return result
    }
  }
}
