package diego.db;

import org.hibernate.boot.model.naming.Identifier;
import org.hibernate.boot.model.naming.PhysicalNamingStrategy;
import org.hibernate.engine.jdbc.env.spi.JdbcEnvironment;

/**
 * A naming strategy to convert camelCase columns and tables to snake_case.
 *
 * I'm keeping this class around in case Hibernate's CamelCaseToSnakeCase strategy doesn't do something I need.
 */
public class SnakeCaseNamingStrategy implements PhysicalNamingStrategy {

    @Override
    public Identifier toPhysicalCatalogName(Identifier name, JdbcEnvironment jdbcEnvironment) {
        return convertToSnakeCase(name);
    }

    @Override
    public Identifier toPhysicalSchemaName(Identifier name, JdbcEnvironment jdbcEnvironment) {
        return convertToSnakeCase(name);
    }

    @Override
    public Identifier toPhysicalTableName(Identifier name, JdbcEnvironment jdbcEnvironment) {
        return convertToSnakeCase(name);
    }

    @Override
    public Identifier toPhysicalSequenceName(Identifier name, JdbcEnvironment jdbcEnvironment) {
        return convertToSnakeCase(name);
    }

    @Override
    public Identifier toPhysicalColumnName(Identifier name, JdbcEnvironment jdbcEnvironment) {
        return convertToSnakeCase(name);
    }

    protected static String convertToSnakeCase(String camelCaseName) {
        final String regex = "([a-z])([A-Z])";
        final String replacement = "$1_$2";
        return camelCaseName.replaceAll(regex, replacement).toLowerCase();
    }

    protected static Identifier convertToSnakeCase(final Identifier identifier) {
        if(identifier == null) return null;
        return Identifier.toIdentifier(convertToSnakeCase((identifier.getText())));
    }
}
