package diego.db

import com.google.inject.Inject
import groovy.util.logging.Slf4j
import org.aopalliance.intercept.MethodInterceptor
import org.aopalliance.intercept.MethodInvocation

/**
 * Intercepts all @Transactional methods and ensures that transactions are grouped and applied correctly.
 */
@Slf4j
class TransactionalMethodInterceptor implements MethodInterceptor {
    private final SessionManager sessionManager

    TransactionalMethodInterceptor(SessionManager sessionManager) {
        this.sessionManager = sessionManager
    }

    @Override
    Object invoke(MethodInvocation invocation) throws Throwable {
        // Apparently you have to return the result of the invocation 🤷🏽‍♂️
        if(sessionManager.session.transaction.active) return invocation.proceed() // If there's a transaction already in progress, just invoke the method.
        // If there is no transaction in progress, begin the transaction and invoke the method
        sessionManager.beginTransaction()
        var result = invocation.proceed()
        sessionManager.commit() // Commit the transaction
        return result
    }
}
