package diego.db.collections;

import java.util.List;
import java.util.Map;

/**
 * A utility class for managing JPQL queries on collections.
 *
 * @author Kevin Sheppard
 * Date: July 06, 2023
 */
public interface CollectionQuery<T> {
    /**
     * @return A single entity from the database.
     */
    T findOne();

    CollectionQuery<T> params(Map<String, Object> params);
    CollectionQuery<T> params(Object... positionalParams);

    /**
     * @return A list of entities from the database
     */
    List<T> findList();

    /**
     * Pagination method to set the limit the amount of rows to return
     *
     * @param maxRows The amount of rows to return
     * @return This instance
     */
    CollectionQuery<T> setMaxRows(int maxRows);

    /**
     * Pagination method to set the first record to return
     * @param firstRow The number of the first row
     * @return This instance
     */
    CollectionQuery<T> setFirstRow(int firstRow);

}
