package diego.db.collections

import org.hibernate.Session
import org.hibernate.query.Query

import javax.transaction.Transactional

/**
 * @author Kevin Sheppard
 * Date: July 07, 2023
 */
class CollectionQueryImpl<T> implements CollectionQuery<T> {
    private Session session
    private Class<T> entityType
    private String jpql
    private Query<T> query

    protected CollectionQueryImpl(Session session, Class<T> entityType, String jpql) {
        this.session = session
        this.entityType = entityType
        this.jpql = jpql
    }

    protected CollectionQueryImpl(Query<T> query) {
        this.query = query
    }

    CollectionQueryImpl<T> params(Map<String, Object> params) {
        for(Map.Entry<String, Object> entry : params.entrySet()) {
            query().setParameter(entry.key, entry.value)
        }
        return this
    }

    CollectionQueryImpl<T> params(Object... positionalParams) {
        for(int i = 0; i<positionalParams.length; i++) {
            query().setParameter(i+1, positionalParams[i])
        }
        return this
    }

    private Query<T> query() {
        if(!query) query = session.createQuery(jpql, entityType)
        return query
    }

    T findOne() {
        return query.singleResult
    }

    List<T> findList() {
        return query.resultList
    }

    CollectionQuery<T> setMaxRows(int maxRows) {
        query.setMaxResults(maxRows)
        return this
    }

    CollectionQuery<T> setFirstRow(int firstRow) {
        query.setFirstResult(firstRow)
        return this
    }
}
