package diego.db.collections

import diego.db.SessionStore
import diego.db.DTO
import groovy.util.logging.Slf4j

import javax.transaction.Transactional

/**
 * This class is very simplified, but the first pass was a major pain in the a**! Thank God it's working and I never want
 * to have to relearn JPA from scratch again!
 *
 * @author Kevin Sheppard
 * Date: July 06, 2023
 */
@Slf4j
class EntityCollectionImpl<T, K extends Serializable> implements EntityCollection<T, K>, SessionStore {
    private Class<T> entityType
    //Not entirely necessary, but it'll keep you from having to peek inside entity classes to determine what type of ID you should supply
    private Class<K> idType

    EntityCollectionImpl() {}

    /*
    * In order to use the TransactionalMethodInterceptor, this class must be created using the injector.
    * In such cases, there must be a way to set these important fields after instantiation
    * */

    EntityCollectionImpl<T, K> forTypes(Class<T> entityType, Class<K> idType) {
        this.entityType = entityType
        this.idType = idType
        return this
    }

    T find(K id) {
        session.find(entityType, id)
    }

    @Transactional
    void delete(K id) {
        final var instance = session.get(entityType, id)
        if(instance) session.delete(instance)
        else log.warn("No ${entityType.name} with ID $id found.")
    }

    @Transactional
    void insert(T instance) {
        session.save(instance)
    }

    @Transactional
    void update(T instance) {
        session.update(instance)
    }

    @Transactional
    void update(K id, DTO dto) {
        final var persistentReference = session.find(entityType, id)
        // Copy the properties from the transient instance to the persistent reference.
        // Since it's persistent, the updates will be applied when the transaction commits.
        if(persistentReference) dto.copyProperties(persistentReference)
    }

    CollectionQuery<T> createQuery(String jpql, Map<String, Object> namedParams) {
        return new CollectionQueryImpl<T>(session, entityType, jpql).params(namedParams)
    }

    CollectionQuery<T> createQuery(String jpql, Object... positionalParams) {
        return new CollectionQueryImpl<T>(session, entityType, jpql).params(positionalParams)
    }

    CollectionQuery<T> createQuery(String jpql) {
        return new CollectionQueryImpl<T>(session.createQuery(jpql, entityType))
    }
}
