package diego.templates

import com.google.inject.Inject
import freemarker.template.Configuration
import freemarker.template.Template

/**
 * A Freemarker template renderer.
 *
 * @author Kevin Sheppard
 * Date: Jul 05, 2023
 */
class FreemarkerTemplateEngine implements TemplateEngine {
    private final Configuration configuration

    @Inject
    FreemarkerTemplateEngine(Configuration configuration) {
        this.configuration = configuration
    }

    @Override
    String render(String templateName) {
        return render(templateName, [:])
    }

    @Override
    String render(String templateName, Map<String, Object> data) {
        final Template template = configuration.getTemplate(templateName)
        final StringWriter writer = new StringWriter()
        template.process(data, writer)
        return writer.toString()
    }
}
