package diego.templates

import io.vertx.core.MultiMap
import io.vertx.core.http.HttpServerRequest

import java.util.function.BiConsumer

/**
 * An implementation of the Request template variable.
 *
 * @author Kevin Sheppard
 * Date: Jul 06, 2023
 */
class RequestModelImplementation implements RequestModel {
    private final HttpServerRequest request
    private final Map<String, Object> params = new LinkedHashMap<>()

    RequestModelImplementation(HttpServerRequest request) {
        this.request = request
        setParams(request.params())
    }

    protected RequestModelImplementation() {
        request = null
    }

    @Override
    String getPath() {
        return request.path()
    }

    void setParams(MultiMap params) {
        for(Map.Entry<String, String> entry : params.entries()) {
            final var key = entry.key
            final var val = entry.value
            if(this.params.containsKey(key)) {
                if(!(this.params[(key)] instanceof List)) {
                    final String currentVal = this.params[(key)]
                    this.params[(key)] = [ currentVal ] as List<String>
                }
                List<String> currentValues = this.params[(key)] as List<String>
                currentValues.add(val)
            } else {
                this.params.put(key, val)
            }
        }
    }

    @Override
    Map<String, Object> getParams() {
        return params
    }
}
