package diego.tooling

import diego.web.ContentTypes
import diego.web.HttpServerResponse
import io.vertx.core.Handler
import io.vertx.ext.web.Router
import io.vertx.ext.web.RoutingContext

/**
 * A development only route that lists all of the application's routes. Useful for debugging.
 *
 * Author: Kevin Sheppard
 * Date: February 25, 2023
 */
class IndexRouteHandler implements Handler<RoutingContext> {
    private final Router router

    IndexRouteHandler(Router router) {
        this.router = router
    }

    @Override
    void handle(RoutingContext event) {
        final var routesString = new StringJoiner("\n")
        router.routes.forEach { route ->
            route.methods().stream()
                    .filter({ route.path != null }) // Exclude filters.
                    .forEach({ method -> routesString.add("${method.name()} ${route.path}") })
        }
        event.response()
                .setStatusCode(HttpServerResponse.HTTP_OK)
                .putHeader("Content-Type", ContentTypes.TEXT_PLAIN).send(routesString.toString())
    }
}
