package diego.utils

/**
 * A trick I learned from the Play Framework to short-circuit application execution in a performant way.
 *
 * @author Kevin Sheppard
 * Date: May 12, 2023
 */
class FastRuntimeException extends RuntimeException {

  FastRuntimeException() {
    super()
  }

  FastRuntimeException(String message) {
    super(message)
  }

  @Override
  /**
   * This is the magic. If you override this method, no stack trace will be generated which is much faster.
   * @return always null
   */
  synchronized Throwable fillInStackTrace() {
    return null
  }

  @Override
  void printStackTrace() {
    println message
  }
}
