package diego.utils

import groovy.util.logging.Slf4j

import java.math.RoundingMode

/**
 * Utility class for monitoring how long the application takes to start.
 *
 * @author Kevin Sheppard
 * Date: Feb 24, 2022
 */
@Slf4j
class Timer {
    private long startTime
    private double elapsedTime
    private static double totalTime

    /**
     * Mark the current time.
     * @return This instance.
     */
    Timer startClock() {
        startTime = System.currentTimeMillis()
        return this
    }

    /**
     * Mark the current time and print how much time has elapsed since starting the clock.
     * @return This instance.
     */
    Timer stopClock() {
        elapsedTime = (System.currentTimeMillis() - startTime)/1000
        totalTime += elapsedTime
        return this
    }

    BigDecimal getElapsedTime() {
        return new BigDecimal(elapsedTime).setScale(1, RoundingMode.HALF_EVEN)
    }

    @SuppressWarnings('GrMethodMayBeStatic')
    BigDecimal getTotalTime() {
        return new BigDecimal(totalTime).setScale(1, RoundingMode.HALF_EVEN)
    }
}
