package diego.utils.httpclient

import org.apache.http.HttpResponse
import org.apache.http.StatusLine
import org.apache.http.util.EntityUtils

/**
 * A response from the HttpClient.
 *
 * @author Kevin Sheppard
 * Date: Jan 05, 2022
 */
class HttpClientResponse {

  private StatusLine statusLine
  private String body

  // Using a factory method to create an instance since it involves processing the entire HttpResponse
  // before it is closed.
  //TODO: Add ability to parse file responses
  protected static HttpClientResponse create(HttpResponse response) {
    final HttpClientResponse clientResponse = new HttpClientResponse()
    clientResponse.statusLine = response.statusLine
    if(response.entity) clientResponse.body = EntityUtils.toString(response.entity)
    return clientResponse
  }

  int getStatusCode() {
    return statusLine.statusCode
  }

  String getStatusMessage() {
    return statusLine.reasonPhrase
  }

  String getBody() {
    return this.body
  }

}
