package diego.utils.httpclient


import org.apache.http.NameValuePair
import org.apache.http.client.entity.UrlEncodedFormEntity
import org.apache.http.entity.ContentType
import org.apache.http.message.BasicNameValuePair

import java.nio.charset.StandardCharsets

/**
 * Creates a URLEncodedFormEntity.
 *
 * @author Kevin Sheppard
 * Date: Jan 06, 2022
 */
class UrlEncodedForm extends FormData {
  private final List<NameValuePair> params = new ArrayList<>()

  /**
   * @param name The name of the form field.
   * @param values One or more values (for checkbox arrays).
   * @return This instance.
   */
  UrlEncodedForm addParameter(String name, String... values) {
    if(values == null) return this
    Arrays.asList(values).stream()
        .filter({ value -> value != null })
        .forEach({ value -> params.add(new BasicNameValuePair(name, value)) })
    return this
  }

  /**
   * Convenience method for adding parameters to the form.
   *
   * @param params The parameters to add to the form.
   * @return This instance.
   */
  UrlEncodedForm withParameters(Map<String, String> params) {
    params.entrySet().forEach( { entry -> addParameter(entry.key, entry.value) })
    return this
  }

  @Override
  ContentType getEncType() {
    return ContentType.APPLICATION_FORM_URLENCODED
  }

  @Override
  protected UrlEncodedFormEntity getEntity() {
    return new UrlEncodedFormEntity(params, StandardCharsets.UTF_8)
  }

}
