package diego.utils.test

import com.google.inject.AbstractModule
import diego.FrameworkModule
import diego.init.Settings

/**
 * A Module that injects the Settings object.
 *
 * Created for performance purposes, so that the user can skip the initialization of the {@link FrameworkModule} if they
 * need to.
 *
 * @author Kevin Sheppard
 * Date: Mar 31, 2022
 */
class ConfigurationModule extends AbstractModule {

  private ConfigurationModule() {}

  @Override
  protected void configure() {
    super.configure()
    bind(Settings).toInstance(Settings.getInstance())
  }

  static ConfigurationModule create() {
    return new ConfigurationModule()
  }

  static ConfigurationModule create(String configurationFile) {
    Settings.load(configurationFile)
    return create()
  }
}
