package diego.utils.test

import com.google.inject.Inject
import com.google.inject.Singleton
import diego.init.Settings
import groovy.util.logging.Slf4j

import java.sql.Connection
import java.sql.DriverManager
import java.sql.SQLException
/**
 *
 *
 * @author Kevin Sheppard
 * Date: Aug 16, 2022
 */
@Singleton
@Slf4j
class SqlScriptRunner {
  private String url, user, password

  @Inject
  SqlScriptRunner(Settings settings) {
    var config = settings.path("app.db")
    url = config.getString("url")
    user = config.getString("user")
    password = config.getString("password")
  }

  /* Allow initializing the class without a dependency container. Useful for unit testing. */
  SqlScriptRunner(String url, String user, String password) {
    this.url = url
    this.user = user
    this.password = password
  }
/**
   * Execute an SQL script file.
   *
   * @param fileName Script file relative to src/test/resources
   */
  public void executeScript(String fileName) {
    Connection connection = null
    try {
      var scriptFile = new File("src/test/resources/$fileName")
      connection = DriverManager.getConnection(url, user, password)
      connection.autoCommit = true
      if(scriptFile.exists()) {
        var buffer = new StringBuilder()
        var scanner = new Scanner(scriptFile)
        while(scanner.hasNextLine()) {
          var line = scanner.nextLine()
          buffer.append(line)
          // If we encounter a semicolon, then that's a complete statement, so run it.
          if(line.endsWith(";")) {
            String command = buffer.toString()
            connection.createStatement().execute(command)
            log.debug(buffer.toString())
            buffer = new StringBuilder()
          } else { // Otherwise, just append a newline and keep scanning the file.
            buffer.append("\n")
          }
        }
      }
      else log.error("src/test/resources/$fileName not found.")
    } catch (SQLException e) {
      log.error(e.message)
    } finally {
      connection?.close()
    }
  }
}
