package diego.validation.constraints

import jakarta.validation.Constraint
import jakarta.validation.Payload

import java.lang.annotation.Documented
import java.lang.annotation.Retention
import java.lang.annotation.Target

import static java.lang.annotation.ElementType.*
import static java.lang.annotation.RetentionPolicy.RUNTIME

/**
 * Ensures that FileUploads are of a particular MIME type. Examples of values include:
 * <pre>
 * image/*
 * application/pdf
 * </pre>
 * @author Kevin Sheppard
 * Date: May 19, 2023
 */
@Target([ FIELD, PARAMETER, ANNOTATION_TYPE ])
@Retention(RUNTIME)
@Constraint(validatedBy = MimeTypeValidator )
@Documented
@interface MimeType {
  String message() default "{messages.mimetype}";

  String[] value();

  Class<?>[] groups() default [];

  Class<? extends Payload>[] payload() default [];
}