package diego.validation.constraints

import io.vertx.ext.web.FileUpload
import jakarta.validation.ConstraintValidator
import jakarta.validation.ConstraintValidatorContext

import java.util.regex.Pattern

/**
 * @author Kevin Sheppard
 * Date: May 19, 2023
 */
class MimeTypeValidator implements ConstraintValidator<MimeType, FileUpload> {
  /**
   * Converts an Ant expression to a Regex pattern.
   *
   * @param expression An Ant expression like image/*
   * @return A Pattern based on the Ant expression
   */
  static Pattern strToPattern(String expression) {
    final String regexStr = expression.replaceAll('\\*', '(.*)')
    return Pattern.compile(regexStr)
  }

  final List<Pattern> mimeTypePatterns = new ArrayList<>()

  @Override
  void initialize(MimeType constraintAnnotation) {
    for(String expression : constraintAnnotation.value()) {
      mimeTypePatterns.add(strToPattern(expression))
    }
  }

  @Override
  boolean isValid(FileUpload value, ConstraintValidatorContext context) {
    if(!value) return false
    Optional<Pattern> matchingPattern = mimeTypePatterns.stream()
        .filter(pattern -> pattern.matcher(value.contentType()).matches())
        .findFirst()
    return matchingPattern.present
  }
}
