package diego.web;

import diego.web.impl.CookieImpl;

/**
 * @author Kevin Sheppard
 * Date: Jan 10, 2022
 */
public interface Cookie {

  public static final long DURATION_ONE_HOUR = 3600;
  public static final long DURATION_ONE_DAY = DURATION_ONE_HOUR * 24;
  public static final long DURATION_ONE_MONTH = DURATION_ONE_DAY * 30;
  public static final long DURATION_ONE_YEAR = DURATION_ONE_MONTH * 12;

  /**
   * Create a cookie with a default max age of one month.
   *
   * @param name The name of the cookie.
   * @param value The cookie's value.
   * @return A new Cookie.
   */
  static Cookie create(String name, String value) {
    return new CookieImpl(io.vertx.core.http.Cookie.cookie(name, value).setMaxAge(DURATION_ONE_MONTH));
  }

  /**
   * Sets how long the cookie should persist. Useful values can be found in the Cookie.DURATION_*
   * static fields of this class.
   *
   * @param maxAge The maximum age of the cookie. Defaults to {@link Cookie#DURATION_ONE_MONTH}.
   * @return This instance.
   */
  Cookie setMaxAge(long maxAge);

  /**
   * @param path The server path this cookie applies to.
   * @return This instance.
   */
  Cookie setPath(String path);

  /**
   *
   * @param domain The domain for this cookie
   * @return This instance.
   */
  Cookie setDomain(String domain);

  String getName();

  String getValue();

  String getPath();

  String getDomain();

  long getMaxAge();
}
